@@ -11,6 +11,7 @@ from multiprocessing import Process as Multiprocess
 from multiprocessing.connection import Connection
 from typing import (
     Any, Dict, Optional, Union, List)
+from warnings import warn
 
 from vivarium.library.dict_utils import deep_merge
 from vivarium.library.topology import assoc_path, get_in
@@ -114,7 +115,7 @@ class Process(metaclass=abc.ABCMeta):
                 '_updater': 'set'}))
 
         self.parameters.setdefault('time_step', DEFAULT_TIME_STEP)
-        self.schema = None
+        self.schema: Optional[dict] = None
 
     def initial_state(self, config: Optional[dict] = None) -> State:
         """Get initial state in embedded path dictionary.
@@ -246,7 +247,12 @@ class Process(metaclass=abc.ABCMeta):
         return state
 
     def is_deriver(self) -> bool:
-        """Check whether this process is a deriver.
+        """Check whether this process is a :term:`deriver`.
+
+        .. deprecated:: 0.3.14
+           Derivers have been deprecated in favor of :term:`steps`, so
+           please override ``is_step`` instead of ``is_deriver``.
+           Support for Derivers may be removed in a future release.
 
         Returns:
             Whether this process is a deriver. This class always returns
@@ -254,6 +260,23 @@ class Process(metaclass=abc.ABCMeta):
         """
         return False
 
+    def is_step(self) -> bool:
+        """Check whether this process is a :term:`step`.
+
+        Returns:
+            Whether this process is a step. This class always returns
+            ``False``, but subclasses may change this behavior.
+        """
+        method = getattr(self.is_deriver, '__func__', None)
+        if method and method is not Process.is_deriver:
+            # `self` is an instance of a subclass that has overridden
+            # `is_deriver`.
+            warn(
+                'is_deriver() is deprecated. Use is_step() instead.',
+                category=FutureWarning)
+            return self.is_deriver()
+        return False
+
     def get_private_state(self) -> State:
         """Get the process's private state.
 
@@ -323,21 +346,23 @@ class Process(metaclass=abc.ABCMeta):
         return True
 
 
-class Deriver(Process, metaclass=abc.ABCMeta):
-    """Base class for :term:`derivers`."""
-    def initial_state(self, config: Optional[dict] = None) -> State:
-        """Subclasses should override this method.
+class Step(Process, metaclass=abc.ABCMeta):
+    """Base class for steps."""
 
-         Given a config, this method should return the Deriver's initial
-         state.
-         """
-        return {}
+    def __init__(self, parameters: Optional[dict] = None) -> None:
+        parameters = parameters or {}
+        super().__init__(parameters)
 
-    def is_deriver(self) -> bool:
-        """Returns ``True`` to signal this process is a deriver."""
+    def is_step(self) -> bool:
+        """Returns ``True`` to signal that this process is a step."""
         return True
 
 
+#: Deriver is just an alias for :py:class:`Step` now that Derivers have
+#: been deprecated.
+Deriver = Step
+
+
 def _run_update(connection: Connection, process: Process) -> None:
     running = True
 
