@@ -3,15 +3,15 @@ import uuid
 import logging as log
 
 from vivarium.core.process import (
-    Deriver,
-    Composer
+    Deriver
 )
+from vivarium.core.composer import Composer
 from vivarium.core.composition import (
     compose_experiment,
     COMPOSER_KEY,
     PROCESS_OUT_DIR,
 )
-from vivarium.core.experiment import (
+from vivarium.core.engine import (
     pp
 )
 
